@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import abc
+import warnings
 
 import numpy as np
 import astropy.nddata
@@ -8,11 +9,12 @@ import astropy.units as u
 from astropy.utils.misc import InheritDocstrings
 
 from ndcube import utils
+from ndcube.ndcube_sequence import NDCubeSequence
 from ndcube.utils.wcs import wcs_ivoa_mapping
 from ndcube.mixins import NDCubeSlicingMixin, NDCubePlotMixin
 
 
-__all__ = ['NDCubeBase', 'NDCube', 'NDCubeOrdered']
+__all__ = ['NDCubeABC', 'NDCubeBase', 'NDCube', 'NDCubeOrdered']
 
 
 class NDCubeMetaClass(abc.ABCMeta, InheritDocstrings):
@@ -21,7 +23,7 @@ class NDCubeMetaClass(abc.ABCMeta, InheritDocstrings):
     """
 
 
-class NDCubeBase(astropy.nddata.NDData, metaclass=NDCubeMetaClass):
+class NDCubeABC(astropy.nddata.NDData, metaclass=NDCubeMetaClass):
 
     @abc.abstractmethod
     def pixel_to_world(self, *quantity_axis_list):
@@ -85,22 +87,36 @@ class NDCubeBase(astropy.nddata.NDData, metaclass=NDCubeMetaClass):
         pass
 
     @abc.abstractmethod
-    def crop_by_coords(self, min_coord_values, interval_widths):
+    def crop_by_coords(self, lower_corner, interval_widths=None, upper_corner=None, units=None):
         """
         Crops an NDCube given minimum values and interval widths along axes.
 
         Parameters
         ----------
-        min_coord_values: iterable of `astropy.units.Quantity`
+        lower_corner: iterable of `astropy.units.Quantity` or `float`
             The minimum desired values along each relevant axis after cropping
             described in physical units consistent with the NDCube's wcs object.
-            The length of the iterable must equal the number of data dimensions and must
-            have the same order as the data.
+            The length of the iterable must equal the number of data dimensions
+            and must have the same order as the data.
 
-        interval_widths: iterable of `astropy.units.Quantity`
-            The width of the region of interest in each dimension in physical units
-            consistent with the NDCube's wcs object.  The length of the iterable must
-            equal the number of data dimensions and must have the same order as the data.
+        interval_widths: iterable of `astropy.units.Quantity` or `float`
+            The width of the region of interest in each dimension in physical
+            units consistent with the NDCube's wcs object. The length of the
+            iterable must equal the number of data dimensions and must have
+            the same order as the data. This argument will be removed in versions
+            2.0, please use upper_corner argument.
+
+        upper_corner: iterable of `astropy.units.Quantity` or `float`
+            The maximum desired values along each relevant axis after cropping
+            described in physical units consistent with the NDCube's wcs object.
+            The length of the iterable must equal the number of data dimensions
+            and must have the same order as the data.
+
+        units: iterable of `astropy.units.quantity.Quantity`, optionnal
+            If the inputs are set without units, the user must set the units
+            inside this argument as `str`.
+            The length of the iterable must equal the number of data dimensions
+            and must have the same order as the data.
 
         Returns
         -------
@@ -109,7 +125,7 @@ class NDCubeBase(astropy.nddata.NDData, metaclass=NDCubeMetaClass):
         """
 
 
-class NDCube(NDCubeSlicingMixin, NDCubePlotMixin, astropy.nddata.NDArithmeticMixin, NDCubeBase):
+class NDCubeBase(NDCubeSlicingMixin, NDCubeABC):
     """
     Class representing N dimensional cubes.
     Extra arguments are passed on to `~astropy.nddata.NDData`.
@@ -214,9 +230,17 @@ class NDCube(NDCubeSlicingMixin, NDCubePlotMixin, astropy.nddata.NDArithmeticMix
         axes_ctype = []
         for i, axis in enumerate(self.missing_axis):
             if not axis:
-                key = ctype[i]
-                if "-TAN" in key:
-                    key = key[:-4]
+                # Find keys in wcs_ivoa_mapping dict that represent start of CTYPE.
+                # Ensure CTYPE is capitalized.
+                keys = list(filter(lambda key: ctype[i].upper().startswith(key), wcs_ivoa_mapping))
+                # If there are multiple valid keys, raise an error.
+                if len(keys) != 1:
+                    raise ValueError("Non-unique CTYPE key.  Please raise an issue at "
+                                     "https://github.com/sunpy/ndcube/issues citing the"
+                                     "following  CTYPE and non-unique keys: "
+                                     "CTYPE = {0}; keys = {1}".format(ctype[i], keys))
+                else:
+                    key = keys[0]
                 axes_ctype.append(wcs_ivoa_mapping.get(key, default=None))
         return tuple(axes_ctype[::-1])
 
@@ -313,7 +337,10 @@ class NDCube(NDCubeSlicingMixin, NDCubePlotMixin, astropy.nddata.NDArithmeticMix
                 int_axes = np.empty(len(axes), dtype=int)
                 for i, axis in enumerate(axes):
                     if isinstance(axis, int):
-                        int_axes[i] = axis
+                        if axis < 0:
+                            int_axes[i] = n_dimensions + axis
+                        else:
+                            int_axes[i] = axis
                     elif isinstance(axis, str):
                         int_axes[i] = utils.cube.get_axis_number_from_axis_name(
                             axis, world_axis_types)
@@ -401,56 +428,94 @@ class NDCube(NDCubeSlicingMixin, NDCubePlotMixin, astropy.nddata.NDArithmeticMix
                     "value": self._extra_coords_wcs_axis[key]["value"]}
         return result
 
-    def crop_by_coords(self, min_coord_values, interval_widths):
+    def crop_by_coords(self, lower_corner, interval_widths=None, upper_corner=None, units=None):
         # The docstring is defined in NDDataBase
 
-        n_dim = len(self.dimensions)
-        if (len(min_coord_values) != len(interval_widths)) or len(min_coord_values) != n_dim:
-            raise ValueError("min_coord_values and interval_widths must have "
-                             "same number of elements as number of data dimensions.")
-        # Convert coords of lower left corner to pixel units.
-        lower_pixels = self.world_to_pixel(*min_coord_values)
-        upper_pixels = self.world_to_pixel(*[min_coord_values[i] + interval_widths[i]
-                                             for i in range(n_dim)])
-        # Round pixel values to nearest integer.
-        lower_pixels = [int(np.rint(l.value)) for l in lower_pixels]
-        upper_pixels = [int(np.rint(u.value)) for u in upper_pixels]
-        item = tuple([slice(lower_pixels[i], upper_pixels[i]) for i in range(n_dim)])
+        n_dim = self.data.ndim
+        # Raising a value error if the arguments have not the same dimensions.
+        # Calculation of upper_corner with the inputing interval_widths
+        # This part of the code will be removed in version 2.0
+        if interval_widths:
+            warnings.warn(
+                "interval_widths will be removed from the API in version 2.0, please use upper_corner argument.")
+            if upper_corner:
+                raise ValueError("Only one of interval_widths or upper_corner "
+                                 "can be set. Recommend using upper_corner as "
+                                 "interval_widths is deprecated.")
+            if (len(lower_corner) != len(interval_widths)) or (len(lower_corner) != n_dim):
+                raise ValueError("lower_corner and interval_widths must have "
+                                 "same number of elements as number of data "
+                                 "dimensions.")
+            upper_corner = [lower_corner[i] + interval_widths[i] for i in range(n_dim)]
+        # Raising a value error if the arguments have not the same dimensions.
+        if (len(lower_corner) != len(upper_corner)) or (len(lower_corner) != n_dim):
+            raise ValueError("lower_corner and upper_corner must have same"
+                             "number of elements as number of data dimensions.")
+        if units:
+            # Raising a value error if units have not the data dimensions.
+            if len(units) != n_dim:
+                raise ValueError('units must have same number of elements as '
+                                 'number of data dimensions.')
+            # If inputs are not Quantity objects, they are modified into specified units
+            lower_corner = [u.Quantity(lower_corner[i], unit=units[i])
+                            for i in range(self.data.ndim)]
+            upper_corner = [u.Quantity(upper_corner[i], unit=units[i])
+                            for i in range(self.data.ndim)]
+        else:
+            if any([not isinstance(x, u.Quantity) for x in lower_corner + upper_corner]):
+                raise TypeError("lower_corner and interval_widths/upper_corner must be "
+                                "of type astropy.units.Quantity or the units kwarg "
+                                "must be set.")
+        # Get all corners of region of interest.
+        all_world_corners_grid = np.meshgrid(
+            *[u.Quantity([lower_corner[i], upper_corner[i]], unit=lower_corner[i].unit).value
+              for i in range(self.data.ndim)])
+        all_world_corners = [all_world_corners_grid[i].flatten()*lower_corner[i].unit
+                             for i in range(n_dim)]
+        # Convert to pixel coordinates
+        all_pix_corners = self.world_to_pixel(*all_world_corners)
+        # Derive slicing item with which to slice NDCube.
+        # Be sure to round down min pixel and round up + 1 the max pixel.
+        item = tuple([slice(int(np.clip(axis_pixels.value.min(), 0, None)),
+                            int(np.ceil(axis_pixels.value.max()))+1)
+                      for axis_pixels in all_pix_corners])
         return self[item]
 
-    def crop_by_extra_coord(self, min_coord_value, interval_width, coord_name):
+    def crop_by_extra_coord(self, coord_name, min_coord_value, max_coord_value):
         """
         Crops an NDCube given a minimum value and interval width along an extra coord.
 
         Parameters
         ----------
+        coord_name: `str`
+            Name of extra coordinate by which to crop.
+
         min_coord_value: Single value `astropy.units.Quantity`
             The minimum desired value of the extra coord after cropping.
             Unit must be consistent with the extra coord on which cropping is based.
 
-        interval_width: Single value `astropy.units.Quantity`
-            The width of the interval along the extra coord axis in physical units
-            consistent with the extra coord.  Unit must be consistent with the extra
-            coord on which cropping is based.
-
-        extra_coord: `str`
-            Name of extra coordinate.
+        min_coord_value: Single value `astropy.units.Quantity`
+            The maximum desired value of the extra coord after cropping.
+            Unit must be consistent with the extra coord on which cropping is based.
 
         Returns
         -------
         result: `ndcube.NDCube`
 
         """
+        if not isinstance(coord_name, str):
+            raise TypeError("The API for this function has changed. "
+                            "Please give coord_name, min_coord_value, max_coord_value")
         extra_coord_dict = self.extra_coords[coord_name]
         if isinstance(extra_coord_dict["value"], u.Quantity):
             extra_coord_values = extra_coord_dict["value"]
         else:
             extra_coord_values = np.asarray(extra_coord_dict["value"])
         w = np.logical_and(extra_coord_values >= min_coord_value,
-                           extra_coord_values < min_coord_value + interval_width)
+                           extra_coord_values < max_coord_value)
         w = np.arange(len(extra_coord_values))[w]
         item = [slice(None)]*len(self.dimensions)
-        item[extra_coord_dict["axis"]] = slice(w[0], w[1]+1)
+        item[extra_coord_dict["axis"]] = slice(w[0], w[-1]+1)
         return self[tuple(item)]
 
     def __repr__(self):
@@ -464,6 +529,44 @@ Axis Types of NDCube: {axis_type}
 """.format(wcs=self.wcs.__repr__(), lengthNDCube=self.dimensions,
            axis_type=self.world_axis_physical_types))
 
+    def explode_along_axis(self, axis):
+        """
+        Separates slices of NDCubes along a given cube axis into a NDCubeSequence
+        of (N-1)DCubes.
+
+        Parameters
+        ----------
+        axis : `int`
+            The axis along which the data is to be changed.
+
+        Returns
+        -------
+        result : `ndcube_sequence.NDCubeSequence`
+
+        """
+        # If axis is -ve then calculate the axis from the length of the dimensions of one cube
+        if axis < 0:
+            axis = len(self.dimensions) + axis
+        # To store the resultant cube
+        result_cubes = []
+        # All slices are initially initialised as slice(None, None, None)
+        cube_slices = [slice(None, None, None)] * self.data.ndim
+        # Slicing the cube inside result_cube
+        for i in range(self.data.shape[axis]):
+            # Setting the slice value to the index so that the slices are done correctly.
+            cube_slices[axis] = i
+            # Set to None the metadata of sliced cubes.
+            item = tuple(cube_slices)
+            sliced_cube = self[item]
+            sliced_cube.meta = None
+            # Appending the sliced cubes in the result_cube list
+            result_cubes.append(sliced_cube)
+        # Creating a new NDCubeSequence with the result_cubes and common axis as axis
+        return NDCubeSequence(result_cubes, common_axis=axis, meta=self.meta)
+
+class NDCube(NDCubeBase, NDCubePlotMixin, astropy.nddata.NDArithmeticMixin):
+    pass
+
 
 class NDCubeOrdered(NDCube):
     """
