@@ -2,17 +2,17 @@
 '''
 Tests for NDCube
 '''
-from collections import namedtuple
+from collections import OrderedDict
 import datetime
 
 import pytest
-import sunpy.map
 import numpy as np
 import astropy.units as u
 
 from ndcube import NDCube, NDCubeOrdered
 from ndcube.utils.wcs import WCS, _wcs_slicer
 from ndcube.tests import helpers
+from ndcube.ndcube_sequence import NDCubeSequence
 
 # sample data for tests
 # TODO: use a fixture reading from a test file. file TBD.
@@ -59,6 +59,20 @@ data_ordered = np.zeros((2, 4, 3, 2))
 data_ordered[0] = data.transpose()
 data_ordered[1] = data.transpose()
 
+h_rotated = {'CTYPE1': 'HPLN-TAN', 'CUNIT1': 'arcsec', 'CDELT1': 0.4, 'CRPIX1': 0,
+             'CRVAL1': 0, 'NAXIS1': 5,
+             'CTYPE2': 'HPLT-TAN', 'CUNIT2': 'arcsec', 'CDELT2': 0.5, 'CRPIX2': 0,
+             'CRVAL2': 0, 'NAXIS2': 5,
+             'CTYPE3': 'Time    ', 'CUNIT3': 'seconds', 'CDELT3': 0.3, 'CRPIX3': 0,
+             'CRVAL3': 0, 'NAXIS3': 2,
+             'PC1_1': 0.714963912964, 'PC1_2': -0.699137151241, 'PC1_3': 0.0,
+             'PC2_1': 0.699137151241, 'PC2_2': 0.714963912964, 'PC2_3': 0.0,
+             'PC3_1': 0.0, 'PC3_2': 0.0, 'PC3_3': 1.0}
+w_rotated = WCS(header=h_rotated, naxis=3)
+
+data_rotated = np.array([[[1, 2, 3, 4, 6], [2, 4, 5, 3, 1], [0, -1, 2, 4, 2], [3, 5, 1, 2, 0]],
+                         [[2, 4, 5, 1, 3], [1, 5, 2, 2, 4], [2, 3, 4, 0, 5], [0, 1, 2, 3, 4]]])
+
 mask_cubem = data > 0
 mask_cube = data >= 0
 uncertaintym = data
@@ -117,6 +131,15 @@ cubet = NDCube(
                   ('hello', 1, u.Quantity(range(data.shape[1]), unit=u.pix)),
                   ('bye', 2, u.Quantity(range(data.shape[2]), unit=u.pix))])
 
+cube_rotated = NDCube(
+    data_rotated,
+    w_rotated,
+    mask=mask_cube,
+    uncertainty=uncertainty,
+    missing_axis=[False, False, False],
+    extra_coords=[('time', 0, u.Quantity(range(data_rotated.shape[0]), unit=u.pix)),
+                  ('hello', 1, u.Quantity(range(data_rotated.shape[1]), unit=u.pix)),
+                  ('bye', 2, u.Quantity(range(data_rotated.shape[2]), unit=u.pix))])
 
 @pytest.mark.parametrize(
     "test_input,expected,mask,wcs,uncertainty,dimensions,world_axis_physical_types,extra_coords",
@@ -814,24 +837,36 @@ def test_world_to_pixel(test_input, expected):
 
 
 @pytest.mark.parametrize("test_input,expected", [
-    ((cubem, [0.7*u.deg, 1.3e-5*u.deg, 1.02e-9*u.m], [1*u.deg, 1*u.deg, 1.06*u.m]), cubem[:, :2])])
+    ((cubem, [0.7*u.deg, 1.3e-5*u.deg, 1.02e-9*u.m], [1*u.deg, 1*u.deg, 4.e-11*u.m], None),
+     cubem[:, :, :3]),
+    ((cube_rotated, [0*u.s, 1.5*u.arcsec, 0*u.arcsec], [1*u.s, 1*u.arcsec, 0.5*u.arcsec], None),
+     cube_rotated[:, :4, 1:5]),
+    ((cubem, [0.7*u.deg, 1.3e-5*u.deg, 1.02e-9*u.m], None, [1.7*u.deg, 1*u.deg, 1.06e-9*u.m]),
+     cubem[:, :, :3]),
+    ((cube_rotated, [0*u.s, 1.5*u.arcsec, 0*u.arcsec], None, [1*u.s, 2.5*u.arcsec, 0.5*u.arcsec]),
+     cube_rotated[:, :4, 1:5]),
+    ((cube_rotated, [0, 1.5, 0], None, [1, 2.5, 0.5], ['s', 'arcsec', 'arcsec']),
+     cube_rotated[:, :4, 1:5])])
 def test_crop_by_coords(test_input, expected):
     helpers.assert_cubes_equal(
         test_input[0].crop_by_coords(*test_input[1:]), expected)
 
 
 @pytest.mark.parametrize("test_input", [
-    (cubem, u.Quantity([0], unit=u.deg), u.Quantity([1.5, 2.], unit=u.deg))])
+    (ValueError, cubem, u.Quantity([0], unit=u.deg), u.Quantity([1.5, 2.], unit=u.deg), None),
+    (ValueError, cubem, [1*u.s], [1*u.s], [1*u.s]),
+    (ValueError, cubem, u.Quantity([0], unit=u.deg), None, u.Quantity([1.5, 2.], unit=u.deg)),
+    (ValueError, cubem, [1], None, [1], ['s', 'deg']),
+    (TypeError, cubem, [1, 2, 3], None, [2, 3, 4])])
 def test_crop_by_coords_error(test_input):
-    with pytest.raises(ValueError):
-        test_input[0].crop_by_coords(*test_input[1:])
+    with pytest.raises(test_input[0]):
+        test_input[1].crop_by_coords(*test_input[2:])
 
 
 @pytest.mark.parametrize(
     "test_input,expected",
-    [((cubem, 0*u.pix, 1.5*u.pix, "time"), cubem[0:2]),
-     ((cube, 0*u.pix, 1.5*u.pix, "bye"), cube[:, :, 0:2]),
-     ((cubet, datetime.datetime(2000, 1, 1), datetime.timedelta(minutes=2), "time"), cubet[:2])])
+    [((cube, "bye", 0*u.pix, 1.5*u.pix), cube[:, :, 0:2]),
+     ((cubet, "bye", 0.5*u.pix, 3.5*u.pix), cubet[:, :, 1:4])])
 def test_crop_by_extra_coord(test_input, expected):
     helpers.assert_cubes_equal(
         test_input[0].crop_by_extra_coord(*tuple(test_input[1:])), expected)
@@ -874,3 +909,21 @@ def test_axis_world_coords_without_input(test_input, expected):
     for i in range(len(all_coords)):
         np.testing.assert_allclose(all_coords[i].value, expected[i].value)
         assert all_coords[i].unit == expected[i].unit
+
+
+@pytest.mark.parametrize("test_input,expected", [
+    ((cubem, 0, 0), ((2*u.pix, 3*u.pix, 4*u.pix), NDCubeSequence, dict, NDCube, OrderedDict)),
+    ((cubem, 1, 0), ((3*u.pix, 2*u.pix, 4*u.pix), NDCubeSequence, dict, NDCube, OrderedDict)),
+    ((cubem, -2, 0), ((3*u.pix, 2*u.pix, 4*u.pix), NDCubeSequence, dict, NDCube, OrderedDict))
+])
+def test_explode_along_axis(test_input, expected):
+    inp_cube, inp_axis, inp_slice = test_input
+    exp_dimensions, exp_type_seq, exp_meta_seq, exp_type_cube, exp_meta_cube = expected
+    output = inp_cube.explode_along_axis(inp_axis)
+    assert tuple(output.dimensions) == tuple(exp_dimensions)
+    assert any(output[inp_slice].dimensions == \
+        u.Quantity((exp_dimensions[1], exp_dimensions[2]), unit='pix'))
+    assert isinstance(output, exp_type_seq)
+    assert isinstance(output[inp_slice], exp_type_cube)
+    assert isinstance(output.meta, exp_meta_seq)
+    assert isinstance(output[inp_slice].meta, exp_meta_cube)
